# -*- coding: latin-1 -*-
"""
Interfaz a los layouts de PySide.
"""

from PySide import QtCore, QtGui

#==========================================================================================================================================
class V(QtGui.QVBoxLayout):
    """
    Acomodacin en vertical.
    """

    #--------------------------------------------------------------------------
    def control( self, control ) :
        """
        Aade un control.
        """
        self.addWidget( control )
        return self

    #--------------------------------------------------------------------------
    def controlc( self, control ) :
        """
        Aade un control.
        """
        self.addWidget( control )
        self.setAlignment(QtCore.Qt.AlignCenter)
        return self

    #--------------------------------------------------------------------------
    def otro( self, layout ) :
        """
        Aade otro layout.
        """
        self.addLayout( layout )
        return self

    #--------------------------------------------------------------------------
    def espacio( self, espacio ) :
        """
        Aade espacio fijo.
        """
        self.addSpacing( espacio )
        return self

    #--------------------------------------------------------------------------
    def margen( self, n ) :
        """
        Margen exterior.
        """
        self.setContentsMargins( n, n, n, n )
        return self

    #--------------------------------------------------------------------------
    def relleno( self, factor ) :
        """
        Aade espacio de relleno, que ocupa lo que puede segn un factor = stretch.
        """
        self.addStretch( factor )
        return self

#==========================================================================================================================================
class H(QtGui.QHBoxLayout):
    """
    Acomodacin en horizontal.
    """

    #--------------------------------------------------------------------------
    def control( self, control ) :
        """
        Aade un control.
        """
        self.addWidget( control )
        return self

    #--------------------------------------------------------------------------
    def otro( self, layout ) :
        """
        Aade otro layout.
        """
        self.addLayout( layout )
        return self

    #--------------------------------------------------------------------------
    def espacio( self, espacio ) :
        """
        Aade espacio fijo.
        """
        self.addSpacing( espacio )
        return self

    #--------------------------------------------------------------------------
    def ponSeparacion( self, tam ) :
        """
        Separacin entre controles
        """
        self.setSpacing( tam )
        return self

    #--------------------------------------------------------------------------
    def margen( self, n ) :
        """
        Margen exterior.
        """
        self.setContentsMargins( n, n, n, n )
        return self

    #--------------------------------------------------------------------------
    def relleno( self, factor ) :
        """
        Aade espacio de relleno, que ocupa lo que puede segn un factor = stretch.
        """
        self.addStretch( factor )
        return self

#==========================================================================================================================================
class G(QtGui.QGridLayout):
    """
    Acomodacin en tabla.
    """

    dicAlineacion = { None:QtCore.Qt.AlignLeft, "d":QtCore.Qt.AlignRight, "c":QtCore.Qt.AlignCenter }

    #--------------------------------------------------------------------------
    def control( self, control, fila, columna, numFilas=1, numColumnas=1, alineacion = None ) :
        """
        Aade un control.
        """
        self.addWidget( control, fila, columna, numFilas, numColumnas, self.dicAlineacion[alineacion] )
        return self

    #--------------------------------------------------------------------------
    def controld( self, control, fila, columna, numFilas=1, numColumnas=1 ) :
        """
        Aade un control.
        """
        self.addWidget( control, fila, columna, numFilas, numColumnas, self.dicAlineacion["d"] )
        return self

    #--------------------------------------------------------------------------
    def controlc( self, control, fila, columna, numFilas=1, numColumnas=1 ) :
        """
        Aade un control.
        """
        self.addWidget( control, fila, columna, numFilas, numColumnas, self.dicAlineacion["c"] )
        return self

    #--------------------------------------------------------------------------
    def otro( self, layout, fila, columna, numFilas=1, numColumnas=1, alineacion = None ) :
        """
        Aade otro layout.
        """
        self.addLayout( layout, fila, columna, numFilas, numColumnas, self.dicAlineacion[alineacion] )
        return self

    #--------------------------------------------------------------------------
    def otroc( self, layout, fila, columna, numFilas=1, numColumnas=1 ) :
        """
        Aade otro layout.
        """
        self.addLayout( layout, fila, columna, numFilas, numColumnas, self.dicAlineacion["c"] )
        return self

    #--------------------------------------------------------------------------
    def otrod( self, layout, fila, columna, numFilas=1, numColumnas=1 ) :
        """
        Aade otro layout.
        """
        self.addLayout( layout, fila, columna, numFilas, numColumnas, self.dicAlineacion["d"] )
        return self

    #--------------------------------------------------------------------------
    def margen( self, n ) :
        """
        Margen exterior.
        """
        self.setContentsMargins( n, n, n, n )
        return self

    #--------------------------------------------------------------------------
    def rellenoColumna( self, col, factor ) :
        """
        Aade espacio de relleno, que ocupa lo que puede segn un factor = stretch.
        """
        self.setColumnStretch(col, factor)
        return self

#==========================================================================================================================================
