# -*- coding: latin-1 -*-

#==========================================================================================================================================
from PySide import QtCore, QtGui

from Code.Constantes import *
from Code.Traducir import tr

import Iconos
import Colocacion

import WBase
import WChat
import WInformacion
import QTUtil
import QTVarios
#==========================================================================================================================================

#==========================================================================================================================================
class Pantalla(QTVarios.WDialogo):

    #--------------------------------------------------------------------------------------------------------------------------------
    def __init__(self, gestor, owner = None ) :

        self.gestor = gestor

        titulo = ""
        icono = Iconos.Aplicacion64()
        extparam = "main"
        QTVarios.WDialogo.__init__( self, owner, titulo, icono, extparam )

        self.setBackgroundRole(QtGui.QPalette.Light)
        self.setStyleSheet( "QToolButton { padding: 2px;}" )

        self.base = WBase.WBase( self, gestor )
        self.capturas = self.base.capturas
        self.chat = WChat.WChat( self )
        self.chat.hide()
        self.capturas.hide()
        self.siCapturas = False
        self.capturas.hide()
        self.informacionPGN = WInformacion.InformacionPGN( self )
        self.siInformacionPGN = False
        self.informacionPGN.hide()

        self.timer = None

        self.tablero = self.base.tablero

        self.splitter = splitter = QtGui.QSplitter(self)
        splitter.addWidget( self.base )
        splitter.addWidget( self.chat )
        splitter.addWidget( self.informacionPGN )

        ly = Colocacion.H().control( splitter ).margen(0)

        self.setLayout( ly )

        if not self.tablero.siMaximizado :
            self.setFixedHeight( self.tablero.ancho+76 )

    #--------------------------------------------------------------------------------------------------------------------------------
    def closeEvent(self, event): # Cierre con X
        self.guardarVideo()
        if not self.gestor.finalX0( ) :
            event.ignore()
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponGestor(self, gestor ) :
        self.gestor = gestor
        self.base.ponGestor( gestor )

    #--------------------------------------------------------------------------------------------------------------------------------
    def muestra( self ) :

        siMax = self.tablero.siMaximizado
        flags = QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint | QtCore.Qt.WindowMinimizeButtonHint
        if siMax :
            flags |= QtCore.Qt.WindowMaximizeButtonHint

        self.setWindowFlags( flags )
        if siMax:
            self.showMaximized()
        else :
            self.recuperarVideo( siTam = False )
            self.show()

        self.ponTitulo()

        self.tamMinimo = self.size()

    #--------------------------------------------------------------------------------------------------------------------------------
    def resizeEvent(self, QResizeEvent) :
        if self.tablero.siMaximizado :
            nue_h = self.size().height()-84
            tab_h= self.tablero.ancho
            sc = float(nue_h)/float(tab_h)
            self.tablero.scale( sc, sc )
            self.tablero.setFixedSize( nue_h, nue_h )
            self.tablero.ancho = nue_h
            self.tablero.escena.update()
            QTUtil.refreshGUI()

    #--------------------------------------------------------------------------------------------------------------------------------
    def muestraVariantes( self, titulo ) :

        flags = QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint | QtCore.Qt.WindowMinimizeButtonHint

        self.setWindowFlags( flags )

        self.setWindowTitle( titulo )

        self.tamMinimo = self.size()
        self.exec_()

    #--------------------------------------------------------------------------------------------------------------------------------
    def ponTamMinimo( self ) :
        if not self.tablero.siMaximizado :
            self.layout().setSizeConstraint(QtGui.QLayout.SetFixedSize)

    #--------------------------------------------------------------------------------------------------------------------------------
    def ponTamNormal( self ) :
        if not self.tablero.siMaximizado :
            self.layout().setSizeConstraint(QtGui.QLayout.SetMinAndMaxSize)

    #--------------------------------------------------------------------------------------------------------------------------------
    def ponTitulo( self ) :
        titulo = tr( "El ajedrez de Lucas" )
        conf = self.gestor.configuracion
        titulo += " - %s [%d %s]" % ( conf.jugador, conf.puntuacion(), tr( "ABREV_PUNTOS" ) )

        self.setWindowTitle(titulo)
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponRotulo1( self, rotulo ) :
        self.base.ponRotulo1( rotulo )
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponRotulo2( self, rotulo ) :
        self.base.ponRotulo2( rotulo )
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponRevision( self, siponer ) :
        if siponer :
            self.base.lbRevision.show()
        else :
            self.base.lbRevision.hide()
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponActivarTutor( self, siActivar ) :
        self.base.ponActivarTutor( siActivar )

    #--------------------------------------------------------------------------------------------------------------------------------
    def ponToolBar( self, liAcciones ) :
        self.base.ponToolBar( liAcciones )
    #--------------------------------------------------------------------------------------------------------------------------------
    def dameToolBar( self ) :
        return self.base.dameToolBar( )
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponAyudas( self, puntos, siAtras = True ) :
        self.base.ponAyudas( puntos, siAtras )
    #--------------------------------------------------------------------------------------------------------------------------------
    def quitaAyudas( self, siTambienTutorAtras, siAtras = True ) :
        self.base.quitaAyudas( siTambienTutorAtras, siAtras )
    #--------------------------------------------------------------------------------------------------------------------------------
    def habilitaToolbar( self, opcion, siHabilitar ) :
        self.base.habilitaToolbar( opcion, siHabilitar )
    #--------------------------------------------------------------------------------------------------------------------------------
    def mostrarOpcionToolBar( self, opcion, siMostrar ) :
        self.base.mostrarOpcionToolbar( opcion, siMostrar )

    #--------------------------------------------------------------------------------------------------------------------------------
    def pgnRefresh( self, siBlancas ) :
        self.base.pgnRefresh()
        self.base.pgn.gobottom( 2 if siBlancas else 1 )
    #--------------------------------------------------------------------------------------------------------------------------------
    def pgnColocate( self, fil, siBlancas ) :
        col = 1 if siBlancas else 2
        self.base.pgn.goto( fil, col )
    #--------------------------------------------------------------------------------------------------------------------------------
    def pgnPosActual( self ) :
        return self.base.pgn.posActual()
    #--------------------------------------------------------------------------------------------------------------------------------
    def pgnMostrar( self, siMostrar ) :
        return self.base.pgn.setVisible( siMostrar )

    #--------------------------------------------------------------------------------------------------------------------------------
    def refresh( self ) :

        self.update()
        QTUtil.refreshGUI()

        if not self.tablero.siMaximizado :

            r = self.geometry()
            s = self.sizeHint()
            if r.width() != s.width() :
                x = QtCore.QRect( r )
                x.setWidth( s.width() )
                self.setGeometry( x )

    #--------------------------------------------------------------------------------------------------------------------------------
    def activaCapturas( self, siActivar=None ) :
        if siActivar is None :
            self.siCapturas = not self.siCapturas
        else :
            self.siCapturas = siActivar
        self.capturas.setVisible(self.siCapturas)
        if self.siCapturas :
            self.capturas.ponLayout( self.tablero.siBlancasAbajo )
        else :
            self.splitter.setHandleWidth(1)

        self.refresh()
    #--------------------------------------------------------------------------------------------------------------------------------
    def activaInformacionPGN( self, siActivar=None ) :
        if siActivar is None :
            self.siInformacionPGN = not self.siInformacionPGN
        else :
            self.siInformacionPGN = siActivar
        self.informacionPGN.setVisible(self.siInformacionPGN)
        self.refresh()
    #--------------------------------------------------------------------------------------------------------------------------------
    def quitaChat( self ) :
        self.chat.hide()
        self.refresh()

    #--------------------------------------------------------------------------------------------------------------------------------
    def ponCapturas( self, dic, jg, apertura ) :
        self.capturas.pon( dic, jg, apertura )

    #--------------------------------------------------------------------------------------------------------------------------------
    def ponInformacionPGN( self, jg, apertura ) :
        self.informacionPGN.ponJG( jg, apertura )

    #--------------------------------------------------------------------------------------------------------------------------------
    def activaJuego( self, siActivar = True, siReloj = False ) :
        self.base.activaJuego( siActivar, siReloj )
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponDatosReloj( self, bl, rb, ng, rn ) :
        self.base.ponDatosReloj( bl, rb, ng, rn )
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponRelojBlancas( self, tm ) :
        self.base.ponRelojBlancas( tm )
    #--------------------------------------------------------------------------------------------------------------------------------
    def ponRelojNegras( self, tm ) :
        self.base.ponRelojNegras( tm )

    #--------------------------------------------------------------------------------------------------------------------------------
    def iniciaReloj( self, enlace, transicion = 100 ) :
        if self.timer is not None :
            self.timer.stop()
            del self.timer

        self.timer = QtCore.QTimer(self)
        self.connect(self.timer, QtCore.SIGNAL("timeout()"), enlace )
        self.timer.start(transicion)

    #--------------------------------------------------------------------------------------------------------------------------------
    def paraReloj( self ) :
        if self.timer is not None :
            self.timer.stop()
            del self.timer
            self.timer = None

    #--------------------------------------------------------------------------------------------------------------------------------
    def columnas60( self, siPoner, cNivel = "NIVEL" ) :
        self.base.columnas60( siPoner, cNivel )

#==========================================================================================================================================

